@@ -1,4 +1,4 @@
-// Copyright 2009-2020 Intel Corporation
+// Copyright 2009-2021 Intel Corporation
 // SPDX-License-Identifier: Apache-2.0
 
 #include "scene_grid_mesh.h"
@@ -141,13 +141,41 @@ namespace embree
       if (vertices[t].getStride() != vertices[0].getStride())
         throw_RTCError(RTC_ERROR_INVALID_OPERATION,"stride of vertex buffers have to be identical for each time step");
 
+#if defined(EMBREE_SYCL_SUPPORT)
+    
+    /* build quadID_to_primID_xy mapping when hardware ray tracing is supported */
+    DeviceGPU* gpu_device = dynamic_cast<DeviceGPU*>(device);
+    if (gpu_device)
+    {
+      const size_t numQuads = getNumTotalQuads();
+      quadID_to_primID_xy.resize(numQuads);
+      
+      for (uint32_t primID=0, quadID=0; primID<size(); primID++)
+      {
+        const Grid& g = grid(primID);
+        for (ssize_t y=0; y<ssize_t(g.resY)-1; y++)
+          for (ssize_t x=0; x<ssize_t(g.resX)-1; x++)
+            quadID_to_primID_xy[quadID++] = { primID, (uint16_t) x, (uint16_t) y };
+      }
+    }
+
+#endif
+    
     Geometry::commit();
   }
   
-  void GridMesh::addElementsToCount (GeometryCounts & counts) const 
+  void GridMesh::addElementsToCount (GeometryCounts& counts) const 
   {
-    if (numTimeSteps == 1) counts.numGrids += numPrimitives;
-    else                   counts.numMBGrids += numPrimitives;
+    if (numTimeSteps == 1) {
+      counts.numGrids += numPrimitives;
+      for (size_t primID=0; primID<numPrimitives; primID++)
+        counts.numSubGrids += getNumSubGrids(primID);
+    }
+    else {
+      counts.numMBGrids += numPrimitives;
+      for (size_t primID=0; primID<numPrimitives; primID++)
+        counts.numMBSubGrids += getNumSubGrids(primID);
+    }
   }
 
   bool GridMesh::verify() 
@@ -172,85 +200,12 @@ namespace embree
     return true;
   }
   
-  void GridMesh::interpolate(const RTCInterpolateArguments* const args)
-  {
-    unsigned int primID = args->primID;
-    float U = args->u;
-    float V = args->v;
-
-    /* clamp input u,v to [0;1] range */
-    U = max(min(U,1.0f),0.0f);
-    V = max(min(V,1.0f),0.0f);
-    
-    RTCBufferType bufferType = args->bufferType;
-    unsigned int bufferSlot = args->bufferSlot;
-    float* P = args->P;
-    float* dPdu = args->dPdu;
-    float* dPdv = args->dPdv;
-    float* ddPdudu = args->ddPdudu;
-    float* ddPdvdv = args->ddPdvdv;
-    float* ddPdudv = args->ddPdudv;
-    unsigned int valueCount = args->valueCount;
-
-    /* calculate base pointer and stride */
-    assert((bufferType == RTC_BUFFER_TYPE_VERTEX && bufferSlot < numTimeSteps) ||
-           (bufferType == RTC_BUFFER_TYPE_VERTEX_ATTRIBUTE && bufferSlot <= vertexAttribs.size()));
-    const char* src = nullptr; 
-    size_t stride = 0;
-    if (bufferType == RTC_BUFFER_TYPE_VERTEX_ATTRIBUTE) {
-      src    = vertexAttribs[bufferSlot].getPtr();
-      stride = vertexAttribs[bufferSlot].getStride();
-    } else {
-      src    = vertices[bufferSlot].getPtr();
-      stride = vertices[bufferSlot].getStride();
-    }
-
-    const Grid& grid = grids[primID];
-    const int grid_width  = grid.resX-1;
-    const int grid_height = grid.resY-1;
-    const float rcp_grid_width = rcp(float(grid_width));
-    const float rcp_grid_height = rcp(float(grid_height));
-    const int iu = min((int)floor(U*grid_width ),grid_width);
-    const int iv = min((int)floor(V*grid_height),grid_height);
-    const float u = U*grid_width-float(iu);
-    const float v = V*grid_height-float(iv);
-    
-    for (unsigned int i=0; i<valueCount; i+=4)
-    {
-      const size_t ofs = i*sizeof(float);
-      const unsigned int idx0 = grid.startVtxID + (iv+0)*grid.lineVtxOffset + iu;
-      const unsigned int idx1 = grid.startVtxID + (iv+1)*grid.lineVtxOffset + iu;
-      
-      const vbool4 valid = vint4((int)i)+vint4(step) < vint4(int(valueCount));
-      const vfloat4 p0 = vfloat4::loadu(valid,(float*)&src[(idx0+0)*stride+ofs]);
-      const vfloat4 p1 = vfloat4::loadu(valid,(float*)&src[(idx0+1)*stride+ofs]);
-      const vfloat4 p2 = vfloat4::loadu(valid,(float*)&src[(idx1+1)*stride+ofs]);
-      const vfloat4 p3 = vfloat4::loadu(valid,(float*)&src[(idx1+0)*stride+ofs]);
-      const vbool4 left = u+v <= 1.0f;
-      const vfloat4 Q0 = select(left,p0,p2);
-      const vfloat4 Q1 = select(left,p1,p3);
-      const vfloat4 Q2 = select(left,p3,p1);
-      const vfloat4 U  = select(left,u,vfloat4(1.0f)-u);
-      const vfloat4 V  = select(left,v,vfloat4(1.0f)-v);
-      const vfloat4 W  = 1.0f-U-V;
-      
-      if (P) {
-        vfloat4::storeu(valid,P+i,madd(W,Q0,madd(U,Q1,V*Q2)));
-      }
-      if (dPdu) { 
-        assert(dPdu); vfloat4::storeu(valid,dPdu+i,select(left,Q1-Q0,Q0-Q1)*rcp_grid_width);
-        assert(dPdv); vfloat4::storeu(valid,dPdv+i,select(left,Q2-Q0,Q0-Q2)*rcp_grid_height);
-      }
-      if (ddPdudu) { 
-        assert(ddPdudu); vfloat4::storeu(valid,ddPdudu+i,vfloat4(zero));
-        assert(ddPdvdv); vfloat4::storeu(valid,ddPdvdv+i,vfloat4(zero));
-        assert(ddPdudv); vfloat4::storeu(valid,ddPdudv+i,vfloat4(zero));
-      }
-    }
+  void GridMesh::interpolate(const RTCInterpolateArguments* const args) {
+    interpolate_impl<4>(args);
   }
   
 #endif
-  
+
   namespace isa
   {
     GridMesh* createGridMesh(Device* device) {
