@@ -16,15 +16,19 @@ namespace embree
     if (object) object->refInc();
     gsubtype = GTY_SUBTYPE_INSTANCE_LINEAR;
     world2local0 = one;
-    local2world = (AffineSpace3ff*) alignedMalloc(numTimeSteps*sizeof(AffineSpace3ff),16);
+    device->memoryMonitor(numTimeSteps*sizeof(AffineSpace3ff), false);
+    local2world = (AffineSpace3ff*) device->malloc(numTimeSteps*sizeof(AffineSpace3ff),16);
     for (size_t i = 0; i < numTimeSteps; i++)
       local2world[i] = one;
+    device->memoryMonitor(sizeof(*this), false);
   }
 
   Instance::~Instance()
   {
-    alignedFree(local2world);
+    device->free(local2world);
+    device->memoryMonitor(-ssize_t(numTimeSteps*sizeof(AffineSpace3ff)), true);
     if (object) object->refDec();
+    device->memoryMonitor(-sizeof(*this), false);
   }
 
   void Instance::setNumTimeSteps (unsigned int numTimeSteps_in)
@@ -32,7 +36,8 @@ namespace embree
     if (numTimeSteps_in == numTimeSteps)
       return;
 
-    AffineSpace3ff* local2world2 = (AffineSpace3ff*) alignedMalloc(numTimeSteps_in*sizeof(AffineSpace3ff),16);
+    device->memoryMonitor(numTimeSteps_in*sizeof(AffineSpace3ff), false);
+    AffineSpace3ff* local2world2 = (AffineSpace3ff*) device->malloc(numTimeSteps_in*sizeof(AffineSpace3ff),16);
 
     for (size_t i = 0; i < min(numTimeSteps, numTimeSteps_in); i++) {
       local2world2[i] = local2world[i];
@@ -42,7 +47,8 @@ namespace embree
       local2world2[i] = one;
     }
 
-    alignedFree(local2world);
+    device->free(local2world);
+    device->memoryMonitor(-ssize_t(numTimeSteps*sizeof(AffineSpace3ff)), true);
     local2world = local2world2;
 
     Geometry::setNumTimeSteps(numTimeSteps_in);
@@ -98,6 +104,7 @@ namespace embree
 
     local2world[timeStep] = xfm;
     gsubtype = GTY_SUBTYPE_INSTANCE_LINEAR;
+    Geometry::update();
   }
 
   void Instance::setQuaternionDecomposition(const AffineSpace3ff& qd, unsigned int timeStep)
@@ -107,6 +114,7 @@ namespace embree
 
     local2world[timeStep] = qd;
     gsubtype = GTY_SUBTYPE_INSTANCE_QUATERNION;
+    Geometry::update();
   }
 
   AffineSpace3fa Instance::getTransform(float time)
@@ -117,6 +125,17 @@ namespace embree
       return getLocal2World(time);
   }
 
+  AffineSpace3fa Instance::getTransform(size_t i, float time)
+  {
+    if (i != 0)
+      throw_RTCError(RTC_ERROR_INVALID_ARGUMENT, "instance has only primitive 0.");
+
+    if (likely(numTimeSteps <= 1))
+      return getLocal2World();
+    else
+      return getLocal2World(time);
+  }
+
   void Instance::setMask (unsigned mask)
   {
     this->mask = mask;
