@@ -1,4 +1,4 @@
-// Copyright 2009-2020 Intel Corporation
+// Copyright 2009-2021 Intel Corporation
 // SPDX-License-Identifier: Apache-2.0
 
 #pragma once
@@ -13,7 +13,7 @@ namespace embree
   {
     template<int M>
     struct UVIdentity {
-      __forceinline void operator() (vfloat<M>& u, vfloat<M>& v) const {}
+      __forceinline void operator() (vfloat<M>& u, vfloat<M>& v, Vec3vf<M>& Ng) const {}
     };
 
 
@@ -21,12 +21,12 @@ namespace embree
     struct Intersect1Epilog1
     {
       RayHit& ray;
-      IntersectContext* context;
+      RayQueryContext* context;
       const unsigned int geomID;
       const unsigned int primID;
 
       __forceinline Intersect1Epilog1(RayHit& ray,
-                                      IntersectContext* context,
+                                      RayQueryContext* context,
                                       const unsigned int geomID,
                                       const unsigned int primID)
         : ray(ray), context(context), geomID(geomID), primID(primID) {}
@@ -63,7 +63,10 @@ namespace embree
         ray.v = hit.v;
         ray.primID = primID;
         ray.geomID = geomID;
-        instance_id_stack::copy(context->user->instID, ray.instID);
+        instance_id_stack::copy_UU(context->user->instID, ray.instID);
+#if defined(RTC_GEOMETRY_INSTANCE_ARRAY)
+        instance_id_stack::copy_UU(context->user->instPrimID, ray.instPrimID);
+#endif
         return true;
       }
     };
@@ -72,12 +75,12 @@ namespace embree
     struct Occluded1Epilog1
     {
       Ray& ray;
-      IntersectContext* context;
+      RayQueryContext* context;
       const unsigned int geomID;
       const unsigned int primID;
 
       __forceinline Occluded1Epilog1(Ray& ray,
-                                     IntersectContext* context,
+                                     RayQueryContext* context,
                                      const unsigned int geomID,
                                      const unsigned int primID)
         : ray(ray), context(context), geomID(geomID), primID(primID) {}
@@ -117,12 +120,12 @@ namespace embree
     {
       RayHitK<K>& ray;
       size_t k;
-      IntersectContext* context;
+      RayQueryContext* context;
       const unsigned int geomID;
       const unsigned int primID;
 
       __forceinline Intersect1KEpilog1(RayHitK<K>& ray, size_t k,
-                                       IntersectContext* context,
+                                       RayQueryContext* context,
                                        const unsigned int geomID,
                                        const unsigned int primID)
         : ray(ray), k(k), context(context), geomID(geomID), primID(primID) {}
@@ -162,7 +165,10 @@ namespace embree
         ray.v[k] = hit.v;
         ray.primID[k] = primID;
         ray.geomID[k] = geomID;
-        instance_id_stack::copy<const unsigned*, vuint<K>*, const size_t&>(context->user->instID, ray.instID, k);
+        instance_id_stack::copy_UV<K>(context->user->instID, ray.instID, k);
+#if defined(RTC_GEOMETRY_INSTANCE_ARRAY)
+        instance_id_stack::copy_UV<K>(context->user->instPrimID, ray.instPrimID, k);
+#endif
         return true;
       }
     };
@@ -172,12 +178,12 @@ namespace embree
     {
       RayK<K>& ray;
       size_t k;
-      IntersectContext* context;
+      RayQueryContext* context;
       const unsigned int geomID;
       const unsigned int primID;
 
       __forceinline Occluded1KEpilog1(RayK<K>& ray, size_t k,
-                                      IntersectContext* context,
+                                      RayQueryContext* context,
                                       const unsigned int geomID,
                                       const unsigned int primID)
         : ray(ray), k(k), context(context), geomID(geomID), primID(primID) {}
@@ -211,26 +217,25 @@ namespace embree
       }
     };
     
-    template<int M, int Mx, bool filter>
+    template<int M, bool filter>
     struct Intersect1EpilogM
     {
       RayHit& ray;
-      IntersectContext* context;
+      RayQueryContext* context;
       const vuint<M>& geomIDs;
       const vuint<M>& primIDs;
 
       __forceinline Intersect1EpilogM(RayHit& ray,
-                                      IntersectContext* context,
+                                      RayQueryContext* context,
                                       const vuint<M>& geomIDs,
                                       const vuint<M>& primIDs)
         : ray(ray), context(context), geomIDs(geomIDs), primIDs(primIDs) {}
 
       template<typename Hit>
-      __forceinline bool operator() (const vbool<Mx>& valid_i, Hit& hit) const
+      __forceinline bool operator() (const vbool<M>& valid_i, Hit& hit) const
       {
         Scene* scene MAYBE_UNUSED = context->scene;
-        vbool<Mx> valid = valid_i;
-        if (Mx > M) valid &= (1<<M)-1;
+        vbool<M> valid = valid_i;
         hit.finalize();
         size_t i = select_min(valid,hit.vt);
         unsigned int geomID = geomIDs[i];
@@ -287,109 +292,31 @@ namespace embree
         ray.v = uv.y;
         ray.primID = primIDs[i];
         ray.geomID = geomID;
-        instance_id_stack::copy(context->user->instID, ray.instID);
+        instance_id_stack::copy_UU(context->user->instID, ray.instID);
+#if defined(RTC_GEOMETRY_INSTANCE_ARRAY)
+        instance_id_stack::copy_UU(context->user->instPrimID, ray.instPrimID);
+#endif
         return true;
 
       }
     };
 
-#if 0 && defined(__AVX512F__) // do not enable, this reduced frequency for BVH4
     template<int M, bool filter>
-    struct Intersect1EpilogM<M,16,filter>
-    {
-      static const size_t Mx = 16;
-      RayHit& ray;
-      IntersectContext* context;
-      const vuint<M>& geomIDs;
-      const vuint<M>& primIDs;
-
-      __forceinline Intersect1EpilogM(RayHit& ray,
-                                      IntersectContext* context,
-                                      const vuint<M>& geomIDs,
-                                      const vuint<M>& primIDs)
-        : ray(ray), context(context), geomIDs(geomIDs), primIDs(primIDs) {}
-
-      template<typename Hit>
-      __forceinline bool operator() (const vbool<Mx>& valid_i, Hit& hit) const
-      {
-        Scene* MAYBE_UNUSED scene = context->scene;
-        vbool<Mx> valid = valid_i;
-        if (Mx > M) valid &= (1<<M)-1;
-        hit.finalize();
-        size_t i = select_min(valid,hit.vt);
-        unsigned int geomID = geomIDs[i];
-
-        /* intersection filter test */
-#if defined(EMBREE_FILTER_FUNCTION) || defined(EMBREE_RAY_MASK)
-        bool foundhit = false;
-        goto entry;
-        while (true)
-        {
-          if (unlikely(none(valid))) return foundhit;
-          i = select_min(valid,hit.vt);
-
-          geomID = geomIDs[i];
-        entry:
-          Geometry* geometry MAYBE_UNUSED = scene->get(geomID);
-
-#if defined(EMBREE_RAY_MASK)
-          /* goto next hit if mask test fails */
-          if ((geometry->mask & ray.mask) == 0) {
-            clear(valid,i);
-            continue;
-          }
-#endif
-
-#if defined(EMBREE_FILTER_FUNCTION) 
-          /* call intersection filter function */
-          if (filter) {
-            if (unlikely(context->hasContextFilter() || geometry->hasIntersectionFilter())) {
-              const Vec2f uv = hit.uv(i);
-              HitK<1> h(context->user,geomID,primIDs[i],uv.x,uv.y,hit.Ng(i));
-              const float old_t = ray.tfar;
-              ray.tfar = hit.t(i);
-              const bool found = runIntersectionFilter1(geometry,ray,context,h);
-              if (!found) ray.tfar = old_t;
-              foundhit |= found;
-              clear(valid,i);
-              valid &= hit.vt <= ray.tfar; // intersection filters may modify tfar value
-              continue;
-            }
-          }
-#endif
-          break;
-        }
-#endif
-
-        vbool<Mx> finalMask(((unsigned int)1 << i));
-        ray.update(finalMask,hit.vt,hit.vu,hit.vv,hit.vNg.x,hit.vNg.y,hit.vNg.z,geomID,primIDs);
-        instance_id_stack::foreach([&](unsigned level)
-        {
-          ray.instID[level] = context->user->instID[level];
-          return (context->user->instID[level] != RTC_INVALID_GEOMETRY_ID);
-        });
-        return true;
-
-      }
-    };
-#endif    
-    
-    template<int M, int Mx, bool filter>
     struct Occluded1EpilogM
     {
       Ray& ray;
-      IntersectContext* context;
+      RayQueryContext* context;
       const vuint<M>& geomIDs;
       const vuint<M>& primIDs;
 
       __forceinline Occluded1EpilogM(Ray& ray,
-                                     IntersectContext* context,
+                                     RayQueryContext* context,
                                      const vuint<M>& geomIDs,
                                      const vuint<M>& primIDs)
         : ray(ray), context(context), geomIDs(geomIDs), primIDs(primIDs) {}
 
       template<typename Hit>
-      __forceinline bool operator() (const vbool<Mx>& valid_i, Hit& hit) const
+      __forceinline bool operator() (const vbool<M>& valid_i, Hit& hit) const
       {
         Scene* scene MAYBE_UNUSED = context->scene;
         /* intersection filter test */
@@ -397,8 +324,7 @@ namespace embree
         if (unlikely(filter))
           hit.finalize(); /* called only once */
 
-        vbool<Mx> valid = valid_i;
-        if (Mx > M) valid &= (1<<M)-1;
+        vbool<M> valid = valid_i;
         size_t m=movemask(valid);
         goto entry;
         while (true)
@@ -446,12 +372,12 @@ namespace embree
     struct Intersect1EpilogMU
     {
       RayHit& ray;
-      IntersectContext* context;
+      RayQueryContext* context;
       const unsigned int geomID;
       const unsigned int primID;
 
       __forceinline Intersect1EpilogMU(RayHit& ray,
-                                       IntersectContext* context,
+                                       RayQueryContext* context,
                                        const unsigned int geomID,
                                        const unsigned int primID)
         : ray(ray), context(context), geomID(geomID), primID(primID) {}
@@ -506,7 +432,10 @@ namespace embree
         ray.v = uv.y;
         ray.primID = primID;
         ray.geomID = geomID;
-        instance_id_stack::copy(context->user->instID, ray.instID);
+        instance_id_stack::copy_UU(context->user->instID, ray.instID);
+#if defined(RTC_GEOMETRY_INSTANCE_ARRAY)
+        instance_id_stack::copy_UU(context->user->instPrimID, ray.instPrimID);
+#endif
         return true;
       }
     };
@@ -515,12 +444,12 @@ namespace embree
     struct Occluded1EpilogMU
     {
       Ray& ray;
-      IntersectContext* context;
+      RayQueryContext* context;
       const unsigned int geomID;
       const unsigned int primID;
 
       __forceinline Occluded1EpilogMU(Ray& ray,
-                                      IntersectContext* context,
+                                      RayQueryContext* context,
                                       const unsigned int geomID,
                                       const unsigned int primID)
         : ray(ray), context(context), geomID(geomID), primID(primID) {}
@@ -560,13 +489,13 @@ namespace embree
     struct IntersectKEpilogM
     {
       RayHitK<K>& ray;
-      IntersectContext* context;
+      RayQueryContext* context;
       const vuint<M>& geomIDs;
       const vuint<M>& primIDs;
       const size_t i;
 
       __forceinline IntersectKEpilogM(RayHitK<K>& ray,
-                                      IntersectContext* context,
+                                      RayQueryContext* context,
                                      const vuint<M>& geomIDs,
                                      const vuint<M>& primIDs,
                                      size_t i)
@@ -616,7 +545,10 @@ namespace embree
         vfloat<K>::store(valid,&ray.v,v);
         vuint<K>::store(valid,&ray.primID,primID);
         vuint<K>::store(valid,&ray.geomID,geomID);
-        instance_id_stack::copy<const unsigned*, vuint<K>*, const vbool<K>&>(context->user->instID, ray.instID, valid);
+        instance_id_stack::copy_UV<K>(context->user->instID, ray.instID, valid);
+#if defined(RTC_GEOMETRY_INSTANCE_ARRAY)
+        instance_id_stack::copy_UV<K>(context->user->instPrimID, ray.instPrimID, valid);
+#endif
         return valid;
       }
     };
@@ -626,14 +558,14 @@ namespace embree
     {
       vbool<K>& valid0;
       RayK<K>& ray;
-      IntersectContext* context;
+      RayQueryContext* context;
       const vuint<M>& geomIDs;
       const vuint<M>& primIDs;
       const size_t i;
 
       __forceinline OccludedKEpilogM(vbool<K>& valid0,
                                      RayK<K>& ray,
-                                     IntersectContext* context,
+                                     RayQueryContext* context,
                                      const vuint<M>& geomIDs,
                                      const vuint<M>& primIDs,
                                      size_t i)
@@ -646,8 +578,8 @@ namespace embree
 
         /* ray masking test */
         Scene* scene MAYBE_UNUSED = context->scene;
-        const unsigned int geomID = geomIDs[i];
-        const unsigned int primID = primIDs[i];
+        const unsigned int geomID MAYBE_UNUSED = geomIDs[i];
+        const unsigned int primID MAYBE_UNUSED = primIDs[i];
         Geometry* geometry MAYBE_UNUSED = scene->get(geomID);
 #if defined(EMBREE_RAY_MASK)
         valid &= (geometry->mask & ray.mask) != 0;
@@ -681,12 +613,12 @@ namespace embree
     struct IntersectKEpilogMU
     {
       RayHitK<K>& ray;
-      IntersectContext* context;
+      RayQueryContext* context;
       const unsigned int geomID;
       const unsigned int primID;
 
       __forceinline IntersectKEpilogMU(RayHitK<K>& ray,
-                                       IntersectContext* context,
+                                       RayQueryContext* context,
                                        const unsigned int geomID,
                                        const unsigned int primID)
         : ray(ray), context(context), geomID(geomID), primID(primID) {}
@@ -731,8 +663,10 @@ namespace embree
         vfloat<K>::store(valid,&ray.v,v);
         vuint<K>::store(valid,&ray.primID,primID);
         vuint<K>::store(valid,&ray.geomID,geomID);
-        instance_id_stack::copy<const unsigned*, vuint<K>*, const vbool<K>&>(context->user->instID, ray.instID, valid);
-
+        instance_id_stack::copy_UV<K>(context->user->instID, ray.instID, valid);
+#if defined(RTC_GEOMETRY_INSTANCE_ARRAY)
+        instance_id_stack::copy_UV<K>(context->user->instPrimID, ray.instPrimID, valid);
+#endif
         return valid;
       }
     };
@@ -742,13 +676,13 @@ namespace embree
     {
       vbool<K>& valid0;
       RayK<K>& ray;
-      IntersectContext* context;
+      RayQueryContext* context;
       const unsigned int geomID;
       const unsigned int primID;
 
       __forceinline OccludedKEpilogMU(vbool<K>& valid0,
                                       RayK<K>& ray,
-                                      IntersectContext* context,
+                                      RayQueryContext* context,
                                       const unsigned int geomID,
                                       const unsigned int primID)
         : valid0(valid0), ray(ray), context(context), geomID(geomID), primID(primID) {}
@@ -788,28 +722,27 @@ namespace embree
       }
     };
     
-    template<int M, int Mx, int K, bool filter>
+    template<int M, int K, bool filter>
     struct Intersect1KEpilogM
     {
       RayHitK<K>& ray;
       size_t k;
-      IntersectContext* context;
+      RayQueryContext* context;
       const vuint<M>& geomIDs;
       const vuint<M>& primIDs;
 
       __forceinline Intersect1KEpilogM(RayHitK<K>& ray, size_t k,
-                                       IntersectContext* context,
+                                       RayQueryContext* context,
                                        const vuint<M>& geomIDs,
                                        const vuint<M>& primIDs)
         : ray(ray), k(k), context(context), geomIDs(geomIDs), primIDs(primIDs) {}
 
       template<typename Hit>
-      __forceinline bool operator() (const vbool<Mx>& valid_i, Hit& hit) const
+      __forceinline bool operator() (const vbool<M>& valid_i, Hit& hit) const
       {
         Scene* scene MAYBE_UNUSED = context->scene;
-        vbool<Mx> valid = valid_i;
+        vbool<M> valid = valid_i;
         hit.finalize();
-        if (Mx > M) valid &= (1<<M)-1;
         size_t i = select_min(valid,hit.vt);
         assert(i<M);
         unsigned int geomID = geomIDs[i];
@@ -858,9 +791,6 @@ namespace embree
 #endif
         assert(i<M);
         /* update hit information */
-#if 0 && defined(__AVX512F__) // do not enable, this reduced frequency for BVH4
-        ray.updateK(i,k,hit.vt,hit.vu,hit.vv,vfloat<Mx>(hit.vNg.x),vfloat<Mx>(hit.vNg.y),vfloat<Mx>(hit.vNg.z),geomID,vuint<Mx>(primIDs));
-#else
         const Vec2f uv = hit.uv(i);
         ray.tfar[k] = hit.t(i);
         ray.Ng.x[k] = hit.vNg.x[i];
@@ -870,29 +800,31 @@ namespace embree
         ray.v[k] = uv.y;
         ray.primID[k] = primIDs[i];
         ray.geomID[k] = geomID;
-        instance_id_stack::copy<const unsigned*, vuint<K>*, const size_t&>(context->user->instID, ray.instID, k);
+        instance_id_stack::copy_UV<K>(context->user->instID, ray.instID, k);
+#if defined(RTC_GEOMETRY_INSTANCE_ARRAY)
+        instance_id_stack::copy_UV<K>(context->user->instPrimID, ray.instPrimID, k);
 #endif
         return true;
       }
     };
     
-    template<int M, int Mx, int K, bool filter>
+    template<int M, int K, bool filter>
     struct Occluded1KEpilogM
     {
       RayK<K>& ray;
       size_t k;
-      IntersectContext* context;
+      RayQueryContext* context;
       const vuint<M>& geomIDs;
       const vuint<M>& primIDs;
 
       __forceinline Occluded1KEpilogM(RayK<K>& ray, size_t k,
-                                      IntersectContext* context,
+                                      RayQueryContext* context,
                                       const vuint<M>& geomIDs,
                                       const vuint<M>& primIDs)
         : ray(ray), k(k), context(context), geomIDs(geomIDs), primIDs(primIDs) {}
 
       template<typename Hit>
-      __forceinline bool operator() (const vbool<Mx>& valid_i, Hit& hit) const
+      __forceinline bool operator() (const vbool<M>& valid_i, Hit& hit) const
       {
         Scene* scene MAYBE_UNUSED = context->scene;
 
@@ -901,8 +833,7 @@ namespace embree
         if (unlikely(filter))
           hit.finalize(); /* called only once */
 
-        vbool<Mx> valid = valid_i;
-        if (Mx > M) valid &= (1<<M)-1;
+        vbool<M> valid = valid_i;
         size_t m=movemask(valid);
         goto entry;
         while (true)
@@ -950,12 +881,12 @@ namespace embree
     {
       RayHitK<K>& ray;
       size_t k;
-      IntersectContext* context;
+      RayQueryContext* context;
       const unsigned int geomID;
       const unsigned int primID;
 
       __forceinline Intersect1KEpilogMU(RayHitK<K>& ray, size_t k,
-                                        IntersectContext* context,
+                                        RayQueryContext* context,
                                         const unsigned int geomID,
                                         const unsigned int primID)
         : ray(ray), k(k), context(context), geomID(geomID), primID(primID) {}
@@ -1002,10 +933,6 @@ namespace embree
 #endif
 
         /* update hit information */
-#if 0 && defined(__AVX512F__) // do not enable, this reduced frequency for BVH4
-        const Vec3fa Ng = hit.Ng(i);
-        ray.updateK(i,k,hit.vt,hit.vu,hit.vv,vfloat<M>(Ng.x),vfloat<M>(Ng.y),vfloat<M>(Ng.z),geomID,vuint<M>(primID));
-#else
         const Vec2f uv = hit.uv(i);
         const Vec3fa Ng = hit.Ng(i);
         ray.tfar[k] = hit.t(i);
@@ -1016,7 +943,9 @@ namespace embree
         ray.v[k] = uv.y;
         ray.primID[k] = primID;
         ray.geomID[k] = geomID;
-        instance_id_stack::copy<const unsigned*, vuint<K>*, const size_t&>(context->user->instID, ray.instID, k);
+        instance_id_stack::copy_UV<K>(context->user->instID, ray.instID, k);
+#if defined(RTC_GEOMETRY_INSTANCE_ARRAY)
+        instance_id_stack::copy_UV<K>(context->user->instPrimID, ray.instPrimID, k);
 #endif
         return true;
       }
@@ -1027,12 +956,12 @@ namespace embree
     {
       RayK<K>& ray;
       size_t k;
-      IntersectContext* context;
+      RayQueryContext* context;
       const unsigned int geomID;
       const unsigned int primID;
 
       __forceinline Occluded1KEpilogMU(RayK<K>& ray, size_t k,
-                                       IntersectContext* context,
+                                       RayQueryContext* context,
                                        const unsigned int geomID,
                                        const unsigned int primID)
         : ray(ray), k(k), context(context), geomID(geomID), primID(primID) {}
