@@ -285,7 +285,7 @@ ze_result_t zeDeviceGetRTASPropertiesExp( const ze_device_handle_t hDevice, ze_r
   if (vendor_id != 0x8086) return ZE_RESULT_ERROR_UNKNOWN;
   
   /* disabling of device check through env variable */
-  const char* disable_device_check = getenv("EMBREE_DISABLE_DEVICEID_CHECK");
+  const char* disable_device_check = std::getenv("EMBREE_DISABLE_DEVICEID_CHECK");
   if (disable_device_check && strcmp(disable_device_check,"1") == 0) {
     pProperties->rtasFormat = (ze_rtas_format_exp_t) ZE_RTAS_DEVICE_FORMAT_EXP_VERSION_1;
     return ZE_RESULT_SUCCESS;
@@ -307,8 +307,10 @@ ze_result_t zeDeviceGetRTASPropertiesExp( const ze_device_handle_t hDevice, ze_r
   /* PVC */
   const bool pvc =
     (0x0BD5 <= device_id && device_id <= 0x0BDB) ||
-    (device_id == 0x0B69                       );
-  
+    (device_id == 0x0B69) ||
+    (device_id == 0x0B6E) ||
+    (device_id == 0x0BD4);
+      
   if (pvc) {
     pProperties->rtasFormat = (ze_rtas_format_exp_t) ZE_RTAS_DEVICE_FORMAT_EXP_VERSION_1;
     return ZE_RESULT_SUCCESS;
