@@ -74,7 +74,7 @@ __forceinline Vec3f intel_get_hit_triangle_normal(intel_ray_query_t& query, inte
   return cross(v1-v0, v2-v0);
 }
 
-__forceinline bool intersect_user_geometry(intel_ray_query_t& query, RayHit& ray, UserGeometry* geom, Scene* scenes[RTC_MAX_INSTANCE_LEVEL_COUNT+1], sycl::private_ptr<RayQueryContext> context, uint32_t geomID, uint32_t primID)
+__forceinline bool intersect_user_geometry(intel_ray_query_t& query, RayHit& ray, UserGeometry* geom, Scene* scenes[RTC_MAX_INSTANCE_LEVEL_COUNT], sycl::private_ptr<RayQueryContext> context, uint32_t geomID, uint32_t primID)
 {
   /* perform ray mask test */
 #if defined(EMBREE_RAY_MASK)
@@ -87,7 +87,11 @@ __forceinline bool intersect_user_geometry(intel_ray_query_t& query, RayHit& ray
   if (!forward_scene) return ishit;
 
   /* forward ray to instanced scene */
-  unsigned int bvh_level = intel_get_hit_bvh_level( query, intel_hit_type_potential_hit ) + 1;
+#if RTC_MAX_INSTANCE_LEVEL_COUNT > 1
+  unsigned int bvh_level = intel_get_hit_bvh_level( query, intel_hit_type_potential_hit );
+#else
+  constexpr unsigned int bvh_level = 0;
+#endif
   Scene* scene = (Scene*) forward_scene;
   scenes[bvh_level] = scene;
   
@@ -106,16 +110,13 @@ __forceinline bool intersect_user_geometry(intel_ray_query_t& query, RayHit& ray
   raydesc.flags |= intel_ray_flags_cull_back_facing_triangles;
 #endif
 
-  intel_raytracing_acceleration_structure_t hwaccel_ptr = (intel_raytracing_acceleration_structure_t) scene->hwaccel.data();
+  intel_raytracing_acceleration_structure_t hwaccel_ptr = (intel_raytracing_acceleration_structure_t) scene->getHWAccel(0);
 
-  EmbreeHWAccel* hwaccel = (EmbreeHWAccel*) hwaccel_ptr;
-  hwaccel_ptr = (intel_raytracing_acceleration_structure_t) hwaccel->AccelTable[0];
-    
   intel_ray_query_forward_ray(query, raydesc, hwaccel_ptr);
   return false;
 }
 
-__forceinline bool intersect_user_geometry(intel_ray_query_t& query, Ray& ray, UserGeometry* geom, Scene* scenes[RTC_MAX_INSTANCE_LEVEL_COUNT+1], sycl::private_ptr<RayQueryContext> context, uint32_t geomID, uint32_t primID)
+__forceinline bool intersect_user_geometry(intel_ray_query_t& query, Ray& ray, UserGeometry* geom, Scene* scenes[RTC_MAX_INSTANCE_LEVEL_COUNT], sycl::private_ptr<RayQueryContext> context, uint32_t geomID, uint32_t primID)
 {
   /* perform ray mask test */
 #if defined(EMBREE_RAY_MASK)
@@ -128,7 +129,11 @@ __forceinline bool intersect_user_geometry(intel_ray_query_t& query, Ray& ray, U
   if (!forward_scene) return ishit;
 
   /* forward ray to instanced scene */
-  unsigned int bvh_level = intel_get_hit_bvh_level( query, intel_hit_type_potential_hit ) + 1;
+#if RTC_MAX_INSTANCE_LEVEL_COUNT > 1
+  unsigned int bvh_level = intel_get_hit_bvh_level( query, intel_hit_type_potential_hit );
+#else
+  constexpr unsigned int bvh_level = 0;
+#endif
   Scene* scene = (Scene*) forward_scene;
   scenes[bvh_level] = scene;
   
@@ -147,20 +152,17 @@ __forceinline bool intersect_user_geometry(intel_ray_query_t& query, Ray& ray, U
   raydesc.flags |= intel_ray_flags_cull_back_facing_triangles;
 #endif
 
-  intel_raytracing_acceleration_structure_t hwaccel_ptr = (intel_raytracing_acceleration_structure_t) scene->hwaccel.data();
-
-  EmbreeHWAccel* hwaccel = (EmbreeHWAccel*) hwaccel_ptr;
-  hwaccel_ptr = (intel_raytracing_acceleration_structure_t) hwaccel->AccelTable[0];
+  intel_raytracing_acceleration_structure_t hwaccel_ptr = (intel_raytracing_acceleration_structure_t) scene->getHWAccel(0);
 
   intel_ray_query_forward_ray(query, raydesc, hwaccel_ptr);
   return false;
 }
 
 template<typename Ray>
-__forceinline bool intersect_instance(intel_ray_query_t& query, Ray& ray, Instance* instance, Scene* scenes[RTC_MAX_INSTANCE_LEVEL_COUNT+1], sycl::private_ptr<RayQueryContext> context, uint32_t geomID, uint32_t primID);
+__forceinline bool intersect_instance(intel_ray_query_t& query, Ray& ray, Instance* instance, Scene* scenes[RTC_MAX_INSTANCE_LEVEL_COUNT], sycl::private_ptr<RayQueryContext> context, uint32_t geomID, uint32_t primID);
 
 template<>
-__forceinline bool intersect_instance(intel_ray_query_t& query, RayHit& ray, Instance* instance, Scene* scenes[RTC_MAX_INSTANCE_LEVEL_COUNT+1], sycl::private_ptr<RayQueryContext> context, uint32_t geomID, uint32_t primID)
+__forceinline bool intersect_instance(intel_ray_query_t& query, RayHit& ray, Instance* instance, Scene* scenes[RTC_MAX_INSTANCE_LEVEL_COUNT], sycl::private_ptr<RayQueryContext> context, uint32_t geomID, uint32_t primID)
 {
   /* perform ray mask test */
 #if defined(EMBREE_RAY_MASK)
@@ -171,7 +173,11 @@ __forceinline bool intersect_instance(intel_ray_query_t& query, RayHit& ray, Ins
   if (!instance_id_stack::push(context->user, geomID, 0))
     return false;
 
-  unsigned int bvh_level = intel_get_hit_bvh_level( query, intel_hit_type_potential_hit ) + 1;
+#if RTC_MAX_INSTANCE_LEVEL_COUNT > 1
+  unsigned int bvh_level = intel_get_hit_bvh_level( query, intel_hit_type_potential_hit );
+#else
+  constexpr unsigned int bvh_level = 0;
+#endif
 
   Scene* object = (Scene*) instance->object;
   const AffineSpace3fa world2local = instance->getWorld2Local(ray.time());
@@ -194,16 +200,15 @@ __forceinline bool intersect_instance(intel_ray_query_t& query, RayHit& ray, Ins
   raydesc.flags |= intel_ray_flags_cull_back_facing_triangles;
 #endif
 
-  intel_raytracing_acceleration_structure_t hwaccel_ptr = (intel_raytracing_acceleration_structure_t) object->hwaccel.data();
   
   uint32_t bvh_id = 0;
-  EmbreeHWAccel* hwaccel = (EmbreeHWAccel*) hwaccel_ptr;
   if (context->args->feature_mask & RTC_FEATURE_FLAG_MOTION_BLUR) {
     float time = clamp(ray.time(),0.0f,1.0f);
-    bvh_id = (uint32_t) clamp(uint32_t(hwaccel->numTimeSegments*time), 0u, hwaccel->numTimeSegments-1);
+    uint32_t numTimeSegments = object->getMaxTimeSegments();
+    bvh_id = (uint32_t) clamp(uint32_t(numTimeSegments*time), 0u, numTimeSegments-1);
   }
 
-  hwaccel_ptr = (intel_raytracing_acceleration_structure_t) hwaccel->AccelTable[bvh_id];
+  intel_raytracing_acceleration_structure_t hwaccel_ptr = (intel_raytracing_acceleration_structure_t) object->getHWAccel(bvh_id);
   
   intel_ray_query_forward_ray(query, raydesc, hwaccel_ptr);
 
@@ -211,7 +216,7 @@ __forceinline bool intersect_instance(intel_ray_query_t& query, RayHit& ray, Ins
 }
 
 template<>
-__forceinline bool intersect_instance(intel_ray_query_t& query, Ray& ray, Instance* instance, Scene* scenes[RTC_MAX_INSTANCE_LEVEL_COUNT+1], sycl::private_ptr<RayQueryContext> context, uint32_t geomID, uint32_t primID)
+__forceinline bool intersect_instance(intel_ray_query_t& query, Ray& ray, Instance* instance, Scene* scenes[RTC_MAX_INSTANCE_LEVEL_COUNT], sycl::private_ptr<RayQueryContext> context, uint32_t geomID, uint32_t primID)
 {
   /* perform ray mask test */
 #if defined(EMBREE_RAY_MASK)
@@ -222,7 +227,11 @@ __forceinline bool intersect_instance(intel_ray_query_t& query, Ray& ray, Instan
   if (!instance_id_stack::push(context->user, geomID, 0))
     return false;
 
-  unsigned int bvh_level = intel_get_hit_bvh_level( query, intel_hit_type_potential_hit ) + 1;
+#if RTC_MAX_INSTANCE_LEVEL_COUNT > 1
+  unsigned int bvh_level = intel_get_hit_bvh_level( query, intel_hit_type_potential_hit );
+#else
+  constexpr unsigned int bvh_level = 0;
+#endif
 
   Scene* object = (Scene*) instance->object;
   const AffineSpace3fa world2local = instance->getWorld2Local(ray.time());
@@ -245,16 +254,14 @@ __forceinline bool intersect_instance(intel_ray_query_t& query, Ray& ray, Instan
   raydesc.flags |= intel_ray_flags_cull_back_facing_triangles;
 #endif
 
-  intel_raytracing_acceleration_structure_t hwaccel_ptr = (intel_raytracing_acceleration_structure_t) object->hwaccel.data();
-  
   uint32_t bvh_id = 0;
-  EmbreeHWAccel* hwaccel = (EmbreeHWAccel*) hwaccel_ptr;
   if (context->args->feature_mask & RTC_FEATURE_FLAG_MOTION_BLUR) {
     float time = clamp(ray.time(),0.0f,1.0f);
-    bvh_id = (uint32_t) clamp(uint32_t(hwaccel->numTimeSegments*time), 0u, hwaccel->numTimeSegments-1);
+    uint32_t numTimeSegments = object->getMaxTimeSegments();
+    bvh_id = (uint32_t) clamp(uint32_t(numTimeSegments*time), 0u, numTimeSegments-1);
   }
 
-  hwaccel_ptr = (intel_raytracing_acceleration_structure_t) hwaccel->AccelTable[bvh_id];
+  intel_raytracing_acceleration_structure_t hwaccel_ptr = (intel_raytracing_acceleration_structure_t) object->getHWAccel(bvh_id);
 
   intel_ray_query_forward_ray(query, raydesc, hwaccel_ptr);
 
@@ -262,10 +269,10 @@ __forceinline bool intersect_instance(intel_ray_query_t& query, Ray& ray, Instan
 }
 
 template<typename Ray>
-__forceinline bool intersect_instance_array(intel_ray_query_t& query, Ray& ray, InstanceArray* instance, Scene* scenes[RTC_MAX_INSTANCE_LEVEL_COUNT+1], sycl::private_ptr<RayQueryContext> context, uint32_t geomID, uint32_t primID);
+__forceinline bool intersect_instance_array(intel_ray_query_t& query, Ray& ray, InstanceArray* instance, Scene* scenes[RTC_MAX_INSTANCE_LEVEL_COUNT], sycl::private_ptr<RayQueryContext> context, uint32_t geomID, uint32_t primID);
 
 template<>
-__forceinline bool intersect_instance_array(intel_ray_query_t& query, RayHit& ray, InstanceArray* instance, Scene* scenes[RTC_MAX_INSTANCE_LEVEL_COUNT+1], sycl::private_ptr<RayQueryContext> context, uint32_t geomID, uint32_t primID)
+__forceinline bool intersect_instance_array(intel_ray_query_t& query, RayHit& ray, InstanceArray* instance, Scene* scenes[RTC_MAX_INSTANCE_LEVEL_COUNT], sycl::private_ptr<RayQueryContext> context, uint32_t geomID, uint32_t primID)
 {
   Scene* object = (Scene*) instance->getObject(primID);
   if (!object) return false;
@@ -279,7 +286,11 @@ __forceinline bool intersect_instance_array(intel_ray_query_t& query, RayHit& ra
   if (!instance_id_stack::push(context->user, geomID, primID))
     return false;
 
-  unsigned int bvh_level = intel_get_hit_bvh_level( query, intel_hit_type_potential_hit ) + 1;
+#if RTC_MAX_INSTANCE_LEVEL_COUNT > 1
+  unsigned int bvh_level = intel_get_hit_bvh_level( query, intel_hit_type_potential_hit );
+#else
+  constexpr unsigned int bvh_level = 0;
+#endif
 
   const AffineSpace3fa world2local = instance->getWorld2Local(primID, ray.time());
   const Vec3fa ray_org = xfmPoint (world2local, (Vec3f) ray.org);
@@ -301,16 +312,14 @@ __forceinline bool intersect_instance_array(intel_ray_query_t& query, RayHit& ra
   raydesc.flags |= intel_ray_flags_cull_back_facing_triangles;
 #endif
 
-  intel_raytracing_acceleration_structure_t hwaccel_ptr = (intel_raytracing_acceleration_structure_t) object->hwaccel.data();
-  
   uint32_t bvh_id = 0;
-  EmbreeHWAccel* hwaccel = (EmbreeHWAccel*) hwaccel_ptr;
   if (context->args->feature_mask & RTC_FEATURE_FLAG_MOTION_BLUR) {
     float time = clamp(ray.time(),0.0f,1.0f);
-    bvh_id = (uint32_t) clamp(uint32_t(hwaccel->numTimeSegments*time), 0u, hwaccel->numTimeSegments-1);
+    uint32_t numTimeSegments = object->getMaxTimeSegments();
+    bvh_id = (uint32_t) clamp(uint32_t(numTimeSegments*time), 0u, numTimeSegments-1);
   }
 
-  hwaccel_ptr = (intel_raytracing_acceleration_structure_t) hwaccel->AccelTable[bvh_id];
+  intel_raytracing_acceleration_structure_t hwaccel_ptr = (intel_raytracing_acceleration_structure_t) object->getHWAccel(bvh_id);
   
   intel_ray_query_forward_ray(query, raydesc, hwaccel_ptr);
 
@@ -318,7 +327,7 @@ __forceinline bool intersect_instance_array(intel_ray_query_t& query, RayHit& ra
 }
 
 template<>
-__forceinline bool intersect_instance_array(intel_ray_query_t& query, Ray& ray, InstanceArray* instance, Scene* scenes[RTC_MAX_INSTANCE_LEVEL_COUNT+1], sycl::private_ptr<RayQueryContext> context, uint32_t geomID, uint32_t primID)
+__forceinline bool intersect_instance_array(intel_ray_query_t& query, Ray& ray, InstanceArray* instance, Scene* scenes[RTC_MAX_INSTANCE_LEVEL_COUNT], sycl::private_ptr<RayQueryContext> context, uint32_t geomID, uint32_t primID)
 {
   Scene* object = (Scene*) instance->getObject(primID);
   if (!object) return false;
@@ -332,7 +341,11 @@ __forceinline bool intersect_instance_array(intel_ray_query_t& query, Ray& ray,
   if (!instance_id_stack::push(context->user, geomID, primID))
     return false;
 
-  unsigned int bvh_level = intel_get_hit_bvh_level( query, intel_hit_type_potential_hit ) + 1;
+#if RTC_MAX_INSTANCE_LEVEL_COUNT > 1
+  unsigned int bvh_level = intel_get_hit_bvh_level( query, intel_hit_type_potential_hit );
+#else
+  constexpr unsigned int bvh_level = 0;
+#endif
 
   const AffineSpace3fa world2local = instance->getWorld2Local(primID, ray.time());
   const Vec3fa ray_org = xfmPoint (world2local, (Vec3f) ray.org);
@@ -354,16 +367,14 @@ __forceinline bool intersect_instance_array(intel_ray_query_t& query, Ray& ray,
   raydesc.flags |= intel_ray_flags_cull_back_facing_triangles;
 #endif
 
-  intel_raytracing_acceleration_structure_t hwaccel_ptr = (intel_raytracing_acceleration_structure_t) object->hwaccel.data();
-  
   uint32_t bvh_id = 0;
-  EmbreeHWAccel* hwaccel = (EmbreeHWAccel*) hwaccel_ptr;
   if (context->args->feature_mask & RTC_FEATURE_FLAG_MOTION_BLUR) {
     float time = clamp(ray.time(),0.0f,1.0f);
-    bvh_id = (uint32_t) clamp(uint32_t(hwaccel->numTimeSegments*time), 0u, hwaccel->numTimeSegments-1);
+    uint32_t numTimeSegments = object->getMaxTimeSegments();
+    bvh_id = (uint32_t) clamp(uint32_t(numTimeSegments*time), 0u, numTimeSegments-1);
   }
 
-  hwaccel_ptr = (intel_raytracing_acceleration_structure_t) hwaccel->AccelTable[bvh_id];
+  intel_raytracing_acceleration_structure_t hwaccel_ptr = (intel_raytracing_acceleration_structure_t) object->getHWAccel(bvh_id);
 
   intel_ray_query_forward_ray(query, raydesc, hwaccel_ptr);
 
@@ -371,7 +382,7 @@ __forceinline bool intersect_instance_array(intel_ray_query_t& query, Ray& ray,
 }
 
 template<typename Ray>
-__forceinline bool intersect_primitive(intel_ray_query_t& query, Ray& ray, Scene* scenes[RTC_MAX_INSTANCE_LEVEL_COUNT+1], Geometry* geom, sycl::private_ptr<RayQueryContext> context, uint32_t geomID, uint32_t primID, const RTCFeatureFlags feature_mask)
+__forceinline bool intersect_primitive(intel_ray_query_t& query, Ray& ray, Scene* scenes[RTC_MAX_INSTANCE_LEVEL_COUNT], Geometry* geom, sycl::private_ptr<RayQueryContext> context, uint32_t geomID, uint32_t primID, const RTCFeatureFlags feature_mask)
 {
 #if defined(EMBREE_SYCL_SUPPORT) && defined(__SYCL_DEVICE_ONLY__)
   bool filter = feature_mask & (RTC_FEATURE_FLAG_FILTER_FUNCTION_IN_ARGUMENTS | RTC_FEATURE_FLAG_FILTER_FUNCTION_IN_GEOMETRY);
@@ -381,19 +392,19 @@ __forceinline bool intersect_primitive(intel_ray_query_t& query, Ray& ray, Scene
   }
 
 #if defined(EMBREE_GEOMETRY_USER)
-  if (geom->getType() == Geometry::GTY_USER_GEOMETRY) {
+  if ((feature_mask & RTC_FEATURE_FLAG_USER_GEOMETRY) && (geom->getType() == Geometry::GTY_USER_GEOMETRY)) {
     return intersect_user_geometry(query,ray,(UserGeometry*)geom, scenes, context, geomID, primID);
   }
 #endif
 
 #if defined(EMBREE_GEOMETRY_INSTANCE)
-  if (geom->getTypeMask() & Geometry::MTY_INSTANCE) {
+  if ((feature_mask & RTC_FEATURE_FLAG_INSTANCE) && (geom->getTypeMask() & Geometry::MTY_INSTANCE)) {
     return intersect_instance(query,ray,(Instance*)geom, scenes, context, geomID, primID);
   }
 #endif
 
 #if defined(EMBREE_GEOMETRY_INSTANCE_ARRAY)
-  if (geom->getTypeMask() & Geometry::MTY_INSTANCE_ARRAY) {
+  if ((feature_mask & RTC_FEATURE_FLAG_INSTANCE_ARRAY) && (geom->getTypeMask() & Geometry::MTY_INSTANCE_ARRAY)) {
     return intersect_instance_array(query,ray,(InstanceArray*)geom, scenes, context, geomID, primID);
   }
 #endif
@@ -619,7 +630,7 @@ __forceinline bool commit_potential_hit(intel_ray_query_t& query, Ray& ray) {
 }
 
 template<typename Ray>
-__forceinline void trav_loop(intel_ray_query_t& query, Ray& ray, Scene* scenes[RTC_MAX_INSTANCE_LEVEL_COUNT+1], sycl::private_ptr<RayQueryContext> context, const RTCFeatureFlags feature_mask)
+__forceinline void trav_loop(intel_ray_query_t& query, Ray& ray, Scene* scene0, Scene* scenes[RTC_MAX_INSTANCE_LEVEL_COUNT], sycl::private_ptr<RayQueryContext> context, const RTCFeatureFlags feature_mask)
 {
   while (!intel_is_traversal_done(query))
   {
@@ -639,16 +650,16 @@ __forceinline void trav_loop(intel_ray_query_t& query, Ray& ray, Scene* scenes[R
    
 #if RTC_MAX_INSTANCE_LEVEL_COUNT > 1
     context->user->instStackSize = bvh_level;
-    Scene* scene = scenes[bvh_level];
+    Scene* scene = bvh_level ? scenes[bvh_level - 1] : scene0;
 #else
     const unsigned int instID = intel_get_hit_instance_id(query, intel_hit_type_potential_hit);
     
     /* assume software instancing mode by default (required for rtcForwardRay) */
-    Scene* scene = scenes[bvh_level]; 
+    Scene* scene = bvh_level ? scenes[0] : scene0;
 
     /* if we are in hardware instancing mode and we need to read the scene from the instance */
     if (bvh_level > 0 && instID != RTC_INVALID_GEOMETRY_ID) {
-      Instance* inst = scenes[0]->get<Instance>(instID);
+      Instance* inst = scene0->get<Instance>(instID);
       scene = (Scene*) inst->object;
       context->user->instID[0] = instID;
     }
@@ -689,10 +700,8 @@ __forceinline void trav_loop(intel_ray_query_t& query, Ray& ray, Scene* scenes[R
 SYCL_EXTERNAL __attribute__((always_inline)) void rtcIntersectRTHW(sycl::global_ptr<RTCSceneTy> hscene, sycl::private_ptr<RTCRayQueryContext> ucontext, sycl::private_ptr<RTCRayHit> rayhit_i, sycl::private_ptr<RTCIntersectArguments> args)
 {
   Scene* scene = (Scene*) hscene.get();
-  intel_raytracing_acceleration_structure_t hwaccel_ptr = (intel_raytracing_acceleration_structure_t) scene->hwaccel.data();
 
-  Scene* scenes[RTC_MAX_INSTANCE_LEVEL_COUNT+1];
-  scenes[0] = scene;
+  Scene* scenes[RTC_MAX_INSTANCE_LEVEL_COUNT];
 
   RayQueryContext context(scene, ucontext, args);
 
@@ -737,20 +746,20 @@ SYCL_EXTERNAL __attribute__((always_inline)) void rtcIntersectRTHW(sycl::global_
      raydesc.flags |= intel_ray_flags_force_non_opaque;
 
   uint32_t bvh_id = 0;
-  EmbreeHWAccel* hwaccel = (EmbreeHWAccel*) hwaccel_ptr;
   if (args->feature_mask & RTC_FEATURE_FLAG_MOTION_BLUR) {
     float time = clamp(ray.time(),0.0f,1.0f);
-    bvh_id = (uint32_t) clamp(uint32_t(hwaccel->numTimeSegments*time), 0u, hwaccel->numTimeSegments-1);
+    uint32_t numTimeSegments = scene->getMaxTimeSegments();
+    bvh_id = (uint32_t) clamp(uint32_t(numTimeSegments*time), 0u, numTimeSegments-1);
   }
 
-  hwaccel_ptr = (intel_raytracing_acceleration_structure_t) hwaccel->AccelTable[bvh_id];
+  intel_raytracing_acceleration_structure_t hwaccel_ptr = (intel_raytracing_acceleration_structure_t) scene->getHWAccel(bvh_id);
   
   intel_ray_query_t query = intel_ray_query_init(raydesc, hwaccel_ptr);
   intel_ray_query_start_traversal(query);
   intel_ray_query_sync(query);
   
   if (args->feature_mask & TRAV_LOOP_FEATURES) {
-    trav_loop(query,ray,scenes,&context,args->feature_mask);
+    trav_loop(query,ray,scene,scenes,&context,args->feature_mask);
   }
 
   bool valid = intel_has_committed_hit(query);
@@ -810,10 +819,8 @@ SYCL_EXTERNAL __attribute__((always_inline)) void rtcIntersectRTHW(sycl::global_
 SYCL_EXTERNAL __attribute__((always_inline)) void rtcOccludedRTHW(sycl::global_ptr<RTCSceneTy> hscene, sycl::private_ptr<RTCRayQueryContext> ucontext, sycl::private_ptr<RTCRay> ray_i, sycl::private_ptr<RTCOccludedArguments> args)
 {
   Scene* scene = (Scene*) hscene.get();
-  intel_raytracing_acceleration_structure_t hwaccel_ptr = (intel_raytracing_acceleration_structure_t) scene->hwaccel.data();
 
-  Scene* scenes[RTC_MAX_INSTANCE_LEVEL_COUNT+1];
-  scenes[0] = scene;
+  Scene* scenes[RTC_MAX_INSTANCE_LEVEL_COUNT];
   
   RayQueryContext context(scene, ucontext, args);
   
@@ -841,20 +848,20 @@ SYCL_EXTERNAL __attribute__((always_inline)) void rtcOccludedRTHW(sycl::global_p
      raydesc.flags |= intel_ray_flags_force_non_opaque;
 
   uint32_t bvh_id = 0;
-  EmbreeHWAccel* hwaccel = (EmbreeHWAccel*) hwaccel_ptr;
   if (args->feature_mask & RTC_FEATURE_FLAG_MOTION_BLUR) {
     float time = clamp(ray.time(),0.0f,1.0f);
-    bvh_id = (uint32_t) clamp(uint32_t(hwaccel->numTimeSegments*time), 0u, hwaccel->numTimeSegments-1);
+    uint32_t numTimeSegments = scene->getMaxTimeSegments();
+    bvh_id = (uint32_t) clamp(uint32_t(numTimeSegments*time), 0u, numTimeSegments-1);
   }
 
-  hwaccel_ptr = (intel_raytracing_acceleration_structure_t) hwaccel->AccelTable[bvh_id];
-  
+  intel_raytracing_acceleration_structure_t hwaccel_ptr = (intel_raytracing_acceleration_structure_t) scene->getHWAccel(bvh_id);
+
   intel_ray_query_t query = intel_ray_query_init(raydesc, hwaccel_ptr);
   intel_ray_query_start_traversal(query);
   intel_ray_query_sync(query);
 
   if (args->feature_mask & TRAV_LOOP_FEATURES) {
-    trav_loop(query,ray,scenes,&context,args->feature_mask);
+    trav_loop(query,ray,scene,scenes,&context,args->feature_mask);
   }
   
   if (intel_has_committed_hit(query))
