@@ -102,6 +102,9 @@ def runConfig(config):
   if "klocwork" in config:
     conf.append("-D EMBREE_TESTING_KLOCWORK="+config["klocwork"])
 
+  if "L0RTAS" in config:
+    conf.append("-D EMBREE_SYCL_L0_RTAS_BUILDER="+config["L0RTAS"])
+
   if "package" in config:
     conf.append("-D EMBREE_STACK_PROTECTOR=ON")
 
@@ -371,9 +374,6 @@ def runConfig(config):
     conf.append("-D BUILD_TESTING=ON")
     conf.append("-D EMBREE_TESTING_INSTALL_TESTS=ON")
     conf.append("-D EMBREE_TESTING_PACKAGE=ON")
-    conf.append("-D EMBREE_TUTORIALS_OPENIMAGEIO=OFF")
-    conf.append("-D EMBREE_TUTORIALS_LIBJPEG=OFF")
-    conf.append("-D EMBREE_TUTORIALS_LIBPNG=OFF")
     if OS == "linux" and config["package"] == "ZIP":
       conf.append("-D EMBREE_INSTALL_DEPENDENCIES=ON")
       conf.append("-D EMBREE_BUILD_GLFW_FROM_SOURCE=ON")
