@@ -47,7 +47,7 @@ Vec3f g_sphere_locations[2*g_num_point_queries] = {
   Vec3f( 5.50f,  0.50f,  -6.50f), Vec3f(0.0f),
   Vec3f( 7.25f, -3.00f,  -1.00f), Vec3f(0.0f),
   Vec3f(-0.25f, -0.50f,  -4.25f), Vec3f(0.0f),
-}; // consequtive pairs of (query point, closest point)
+}; // consecutive pairs of (query point, closest point)
 RTCGeometry g_spheres = nullptr;
 RTCGeometry g_lines = nullptr;
 unsigned int g_spheres_geomID = 111111;
@@ -99,7 +99,7 @@ void instanceBoundsFunc(const struct RTCBoundsFunctionArguments* args)
   bounds_o->upper_z = upper.z;
 }
 
-inline void pushInstanceId(RTCIntersectContext* ctx, unsigned int id)
+inline void pushInstanceId(RTCRayQueryContext* ctx, unsigned int id)
 {
 #if RTC_MAX_INSTANCE_LEVEL_COUNT > 1
   ctx->instID[ctx->instStackSize++] = id;
@@ -108,7 +108,7 @@ inline void pushInstanceId(RTCIntersectContext* ctx, unsigned int id)
 #endif
 }
 
-inline void popInstanceId(RTCIntersectContext* ctx)
+inline void popInstanceId(RTCRayQueryContext* ctx)
 {
 #if RTC_MAX_INSTANCE_LEVEL_COUNT > 1
   ctx->instID[--ctx->instStackSize] = RTC_INVALID_GEOMETRY_ID;
@@ -121,7 +121,7 @@ void instanceIntersectFunc(const RTCIntersectFunctionNArguments* args)
 {
   const int* valid = args->valid;
   void* ptr  = args->geometryUserPtr;
-  RTCIntersectContext* context = args->context;
+  RTCRayQueryContext* context = args->context;
   RTCRayHitN* rays = (RTCRayHitN*)args->rayhit;
                                     
   assert(args->N == 1);
@@ -138,9 +138,15 @@ void instanceIntersectFunc(const RTCIntersectFunctionNArguments* args)
   ray->dir = (Vec3ff) xfmVector(instance->world2local,ray_dir);
   ray->tnear() = ray_tnear;
   ray->tfar  = ray_tfar;
+  
+  RTCIntersectArguments iargs;
+  rtcInitIntersectArguments(&iargs);
+  iargs.context = context;
+  
   pushInstanceId(context, instance->userID);
-  rtcIntersect1(instance->object,context,RTCRayHit_(*ray));
+  rtcIntersect1(instance->object,RTCRayHit_(*ray),&iargs);
   popInstanceId(context);
+  
   const float updated_tfar = ray->tfar;
   ray->org = ray_org;
   ray->dir = ray_dir;
@@ -154,7 +160,7 @@ inline void pushInstanceIdAndTransform(RTCPointQueryContext* context,
 {
   context->instID[context->instStackSize] = id;
 
-  // local copies of const references to fullfill alignment constraints
+  // local copies of const references to fulfill alignment constraints
   AffineSpace3fa w2i = w2i_in;
   AffineSpace3fa i2w = i2w_in;
 
@@ -202,7 +208,7 @@ Instance* createInstance (RTCScene scene, RTCScene object, int userID, const Vec
   rtcSetGeometryUserData(instance->geometry,instance);
   rtcSetGeometryBoundsFunction(instance->geometry,instanceBoundsFunc,nullptr);
   rtcSetGeometryIntersectFunction(instance->geometry,instanceIntersectFunc);
-  rtcSetGeometryPointQueryFunction(instance->geometry, instanceClosestPointFunc);
+  rtcSetGeometryPointQueryFunction(instance->geometry,instanceClosestPointFunc);
   rtcCommitGeometry(instance->geometry);
   rtcAttachGeometry(scene,instance->geometry);
   rtcReleaseGeometry(instance->geometry);
@@ -294,14 +300,14 @@ bool closestPointFunc(RTCPointQueryFunctionArguments* args)
   if (stackSize > 0 && args->similarityScale > 0)
   {
     // Instance transform is a similarity transform, therefore we 
-    // can comute distance insformation in instance space. Therefore,
+    // can compute distance insformation in instance space. Therefore,
     // transform query position into local instance space.
     AffineSpace3fa const& m = (*(AffineSpace3fa*)context->world2inst[stackPtr]);
     q = xfmPoint(m, q);
   }
   else if (stackSize > 0)
   {
-    // Instance transform is not a similarity tranform. We have to transform the
+    // Instance transform is not a similarity transform. We have to transform the
     // primitive data into world space and perform distance computations in
     // world space to ensure correctness.
     v0 = xfmPoint(inst2world, v0);
@@ -329,8 +335,8 @@ bool closestPointFunc(RTCPointQueryFunctionArguments* args)
    */
   if (d < args->query->radius)
   {
-    args->query->radius = d;
     ClosestPointResult* result = (ClosestPointResult*)args->userPtr;
+    args->query->radius = d;
     result->p = args->similarityScale > 0 ? xfmPoint(inst2world, p) : p;
     result->primID = primID;
     result->geomID = geomID;
@@ -505,7 +511,7 @@ void updateGeometryAndQueries(float time)
                     * AffineSpace3fa::rotate(Vec3f(0.f, 1.f, 0.f), float(pi)/2.f)
                     * AffineSpace3fa::rotate(Vec3f(1.f, 0.f, 0.f), 0.2f * sin(g_animate_time));
 
-  g_instance_xfm[1] = AffineSpace3fa::translate(Vec3f(0.f, 3.f + 1.5f*sin(g_animate_time), -9.f)) 
+  g_instance_xfm[1] = AffineSpace3fa::translate(Vec3f(0.f, 3.f + 1.5f*sin(g_animate_time), -9.f))
                     * AffineSpace3fa::scale(Vec3fa(1.f, 2.f, 3.f))
                     * AffineSpace3fa::rotate(Vec3f(0.f, 1.f, 0.f), float(pi));
   
@@ -534,14 +540,14 @@ void updateGeometryAndQueries(float time)
   rtcCommitGeometry(g_instanceEmbree[2]);
   rtcCommitScene(g_sceneEmbreeInstance);
   
-  g_instanceUserDefined[0]->local2world = g_instance_xfm[0];
-  g_instanceUserDefined[1]->local2world = g_instance_xfm[1];
-  g_instanceUserDefined[2]->local2world = g_instance_xfm[2];
+  //g_instanceUserDefined[0]->local2world = g_instance_xfm[0];
+  //g_instanceUserDefined[1]->local2world = g_instance_xfm[1];
+  //g_instanceUserDefined[2]->local2world = g_instance_xfm[2];
 
   /* update scene */
-  updateInstance(g_sceneUserDefinedInstance,g_instanceUserDefined[0]);
-  updateInstance(g_sceneUserDefinedInstance,g_instanceUserDefined[1]);
-  updateInstance(g_sceneUserDefinedInstance,g_instanceUserDefined[2]);
+  //updateInstance(g_sceneUserDefinedInstance,g_instanceUserDefined[0]);
+  //updateInstance(g_sceneUserDefinedInstance,g_instanceUserDefined[1]);
+  //updateInstance(g_sceneUserDefinedInstance,g_instanceUserDefined[2]);
   rtcCommitScene(g_sceneUserDefinedInstance);
 
   g_scene = g_userDefinedInstancing ? g_sceneUserDefinedInstance : g_sceneEmbreeInstance;
@@ -600,20 +606,20 @@ extern "C" void device_init (char* cfg)
 
   // add the four objects to all three scenes
   {
-    RTCScene scenes[3] = { g_scene1, g_sceneEmbreeInstance, g_sceneUserDefinedInstance };
+    RTCScene scenes[] = { g_scene1, g_sceneEmbreeInstance, g_sceneUserDefinedInstance };
     for (int m = 0; m < 4; ++m)
-    for (int s = 0; s < 3; ++s) 
+    for (RTCScene scene : scenes)
     {
       RTCGeometry geom = rtcNewGeometry (g_device, RTC_GEOMETRY_TYPE_TRIANGLE);
       rtcSetGeometryPointQueryFunction(geom, closestPointFunc);
       rtcCommitGeometry(geom);
       rtcSetSharedGeometryBuffer(geom,RTC_BUFFER_TYPE_VERTEX,0,RTC_FORMAT_FLOAT3,g_triangle_meshes[m]->vertices, 0,sizeof(Vertex),  g_triangle_meshes[m]->num_vertices);
       rtcSetSharedGeometryBuffer(geom,RTC_BUFFER_TYPE_INDEX, 0,RTC_FORMAT_UINT3, g_triangle_meshes[m]->triangles,0,sizeof(Triangle),g_triangle_meshes[m]->num_triangles);
-      rtcAttachGeometryByID(scenes[s], geom, m);
+      rtcAttachGeometryByID(scene, geom, m);
       rtcReleaseGeometry(geom);
       rtcCommitGeometry(geom);
+      rtcCommitScene(scene);
     }
-    rtcCommitScene(g_scene1);
   }
 
   /* compute bounding box of the scene that will be instanced */
@@ -636,31 +642,31 @@ extern "C" void device_init (char* cfg)
     rtcAttachGeometryByID(g_sceneEmbreeInstance, g_instanceEmbree[i], 4+i);
     rtcReleaseGeometry(g_instanceEmbree[i]);
     rtcCommitGeometry(g_instanceEmbree[i]);
-    
-    g_instanceUserDefined[i] = createInstance(g_sceneUserDefinedInstance, g_scene1, i, bbmin, bbmax);
+
+    //g_instanceUserDefined[i] = createInstance(g_sceneUserDefinedInstance, g_scene1, i, bbmin, bbmax);
   }
 
   {
     // add visualization spheres to both scenes
-    RTCScene scenes[2] = { g_sceneEmbreeInstance, g_sceneUserDefinedInstance };
-    for (unsigned int s = 0; s < 2; ++s)
-    {
-      g_spheres = rtcNewGeometry(g_device, RTC_GEOMETRY_TYPE_SPHERE_POINT);
-      rtcSetSharedGeometryBuffer(g_spheres, RTC_BUFFER_TYPE_VERTEX, 0, RTC_FORMAT_FLOAT4, g_sphere_vertex_buffer, 0, sizeof(Vec4f), 2*g_num_point_queries);
-      rtcAttachGeometryByID(scenes[s], g_spheres, g_spheres_geomID);
-      rtcReleaseGeometry(g_spheres);
-      rtcCommitGeometry(g_spheres);
-      
-      g_lines = rtcNewGeometry(g_device, RTC_GEOMETRY_TYPE_FLAT_LINEAR_CURVE);
-      rtcSetSharedGeometryBuffer(g_lines, RTC_BUFFER_TYPE_VERTEX, 0, RTC_FORMAT_FLOAT4, g_line_vertex_buffer, 0, sizeof(Vec4f), 2*g_num_point_queries);
-      rtcSetSharedGeometryBuffer(g_lines, RTC_BUFFER_TYPE_INDEX,  0, RTC_FORMAT_UINT, g_line_index_buffer, 0, sizeof(unsigned int), g_num_point_queries);
-      rtcAttachGeometryByID(scenes[s], g_lines, g_lines_geomID);
-      rtcReleaseGeometry(g_lines);
-      rtcCommitGeometry(g_lines);
-    }
-  }
+    RTCScene scenes[] = { g_sceneEmbreeInstance, g_sceneUserDefinedInstance };
 
-  updateGeometryAndQueries(0.f);
+    g_spheres = rtcNewGeometry(g_device, RTC_GEOMETRY_TYPE_SPHERE_POINT);
+    rtcSetSharedGeometryBuffer(g_spheres, RTC_BUFFER_TYPE_VERTEX, 0, RTC_FORMAT_FLOAT4, g_sphere_vertex_buffer, 0, sizeof(Vec4f), 2*g_num_point_queries);
+
+    g_lines = rtcNewGeometry(g_device, RTC_GEOMETRY_TYPE_FLAT_LINEAR_CURVE);
+    rtcSetSharedGeometryBuffer(g_lines, RTC_BUFFER_TYPE_VERTEX, 0, RTC_FORMAT_FLOAT4, g_line_vertex_buffer, 0, sizeof(Vec4f), 2*g_num_point_queries);
+    rtcSetSharedGeometryBuffer(g_lines, RTC_BUFFER_TYPE_INDEX,  0, RTC_FORMAT_UINT, g_line_index_buffer, 0, sizeof(unsigned int), g_num_point_queries);
+    rtcCommitGeometry(g_spheres);
+    rtcCommitGeometry(g_lines);
+    for (RTCScene scene : scenes)
+    {
+      rtcAttachGeometryByID(scene, g_spheres, g_spheres_geomID);
+      rtcAttachGeometryByID(scene, g_lines, g_lines_geomID);
+      rtcCommitScene(scene);
+    }
+    rtcReleaseGeometry(g_spheres);
+    rtcReleaseGeometry(g_lines);
+  }
 }
 
 inline Vec3fa face_forward(const Vec3fa& dir, const Vec3fa& _Ng) {
@@ -671,9 +677,6 @@ inline Vec3fa face_forward(const Vec3fa& dir, const Vec3fa& _Ng) {
 /* task that renders a single screen tile */
 Vec3fa renderPixelStandard(float x, float y, const ISPCCamera& camera, RayStats& stats)
 {
-  RTCIntersectContext context;
-  rtcInitIntersectContext(&context);
-  
   /* initialize ray */
   Ray ray(Vec3fa(camera.xfm.p), 
                      Vec3fa(normalize(x*camera.xfm.l.vx + y*camera.xfm.l.vy + camera.xfm.l.vz)), 
@@ -681,7 +684,7 @@ Vec3fa renderPixelStandard(float x, float y, const ISPCCamera& camera, RayStats&
                      RTC_INVALID_GEOMETRY_ID, RTC_INVALID_GEOMETRY_ID);
 
   /* intersect ray with scene */
-  rtcIntersect1(g_scene, &context, RTCRayHit_(ray));
+  rtcIntersect1(g_scene, RTCRayHit_(ray));
   RayStats_addRay(stats);
 
   /* shade pixels */
