@@ -4,7 +4,7 @@
 #pragma once
 
 #include "lights.h"
-#include "../../../include/embree3/rtcore.h"
+#include "../../../include/embree4/rtcore.h"
 RTC_NAMESPACE_USE
 #include "../math/random_sampler.h"
 
@@ -137,11 +137,19 @@ namespace embree
         return empty;
       }
 
+      virtual BBox3fa bounds(size_t i) const {
+        return empty;
+      }
+
       /* calculates linear bounding box of node */
       virtual LBBox3fa lbounds() const {
         return empty;
       }
 
+      virtual LBBox3fa lbounds(size_t i) const {
+        return empty;
+      }
+
       /* calculates number of primitives */
       virtual size_t numPrimitives() const {
         return 0;
@@ -324,6 +332,33 @@ namespace embree
           THROW_RUNTIME_ERROR("number of transformations does not match");
       }
 
+      friend __forceinline std::vector<Transformations> operator* ( const std::vector<Transformations>& a, const Transformations& b )
+      {
+        std::vector<Transformations> result;
+        for (size_t i = 0; i < a.size(); ++i) {
+          result.push_back(a[i] * b);
+        }
+        return result;
+      }
+
+      friend __forceinline std::vector<Transformations> operator* ( const Transformations& a, const std::vector<Transformations>& b )
+      {
+        return b * a;
+      }
+
+      friend __forceinline std::vector<Transformations> operator* ( const std::vector<Transformations>& a, const std::vector<Transformations>& b )
+      {
+        if(a.size() != b.size())
+          THROW_RUNTIME_ERROR("number of transformations does not match");
+
+        std::vector<Transformations> result;
+        for (size_t i = 0; i < a.size(); ++i) {
+          result.push_back(a[i] * b[i]);
+        }
+
+        return result;
+      }
+
       AffineSpace3ff interpolate (const float gtime) const
       {
         assert(time_range.lower == 0.0f && time_range.upper == 1.0f);
@@ -693,7 +728,93 @@ namespace embree
       Transformations spaces;
       Ref<Node> child;
     };
-    
+
+    struct MultiTransformNode : public Node
+    {
+      ALIGNED_STRUCT_(16);
+
+      MultiTransformNode (const avector<AffineSpace3fa>& xfm, const Ref<Node>& child)
+        : child(child)
+      {
+        for (const AffineSpace3fa& space : xfm) {
+          spaces.push_back(Transformations(space));
+        }
+      }
+
+      MultiTransformNode (const avector<AffineSpace3fa>& xfm0, const avector<AffineSpace3fa>& xfm1, const Ref<Node>& child)
+        : child(child)
+      {
+        assert(xfm0.size() == xfm1.size());
+        for (size_t i = 0; i < xfm0.size(); ++i) {
+          spaces.push_back(Transformations(xfm0[i], xfm1[i]));
+        }
+      }
+
+      MultiTransformNode (const avector<avector<AffineSpace3ff>>& spaces_in, const Ref<Node>& child)
+        : child(child)
+      {
+        assert(spaces_in.size() > 0);
+        const size_t time_steps = spaces_in.size();
+        const size_t array_size = spaces_in[0].size();
+        for (size_t i = 0; i < array_size; ++i) {
+          avector<AffineSpace3ff> s;
+          for (size_t j = 0; j < time_steps; ++j) {
+            s.push_back(spaces_in[j][i]);
+          }
+          spaces.push_back(Transformations(s));
+        }
+      }
+
+      MultiTransformNode(const std::vector<Transformations>& spaces, const Ref<Node>& child)
+        : spaces(spaces), child(child) {}
+
+      virtual void setMaterial(Ref<MaterialNode> material) {
+        child->setMaterial(material);
+      }
+
+      virtual void print(std::ostream& cout, int depth);
+
+      virtual void calculateStatistics(Statistics& stat);
+      virtual void calculateInDegree();
+      virtual bool calculateClosed(bool group_instancing);
+      virtual void resetInDegree();
+
+      virtual BBox3fa bounds(size_t i) const {
+        return spaces[i].bounds(child->bounds());
+      }
+
+      virtual LBBox3fa lbounds(size_t i) const {
+        return spaces[i].lbounds(child->lbounds());
+      }
+
+      virtual size_t numPrimitives() const {
+        return child->numPrimitives();
+      }
+
+      virtual AffineSpace3ff get(size_t i, float time) const
+      {
+        if (spaces[i].size() <= 1) return spaces[i][0];
+
+        int numTimeSteps = spaces[i].size();
+
+        BBox1f time_range = spaces[i].time_range;
+        time = frac((time-time_range.lower)/time_range.size());
+        time = (numTimeSteps-1)*time;
+        int   itime = (int)floor(time);
+        itime = min(max(itime,0),(int)numTimeSteps-2);
+        float ftime = time - (float)itime;
+
+        const AffineSpace3ff xfm0 = spaces[i][itime+0];
+        const AffineSpace3ff xfm1 = spaces[i][itime+1];
+        const AffineSpace3ff xfm  = lerp(xfm0,xfm1,ftime);
+        return xfm;
+      }
+
+    public:
+      std::vector<Transformations> spaces;
+      Ref<Node> child;
+    };
+
     struct GroupNode : public Node
     { 
       GroupNode (const size_t N = 0) { 
@@ -722,21 +843,21 @@ namespace embree
       virtual BBox3fa bounds() const
       {
         BBox3fa b = empty;
-        for (auto c : children) b.extend(c->bounds());
+        for (auto& c : children) b.extend(c->bounds());
         return b;
       }
 
       virtual LBBox3fa lbounds() const
       {
         LBBox3fa b = empty;
-        for (auto c : children) b.extend(c->lbounds());
+        for (auto& c : children) b.extend(c->lbounds());
         return b;
       }
 
       virtual size_t numPrimitives() const 
       {
         size_t n = 0;
-        for (auto child : children) n += child->numPrimitives();
+        for (auto& child : children) n += child->numPrimitives();
         return n;
       }
 
@@ -910,7 +1031,7 @@ namespace embree
     
     struct MaterialNode : public Node
     {
-      ALIGNED_STRUCT_(16);
+      ALIGNED_STRUCT_USM_(16)
 
       MaterialNode(const std::string& name = "")
         : Node(name) {}
@@ -970,7 +1091,7 @@ namespace embree
       {
         BBox3fa b = empty;
         for (const auto& p : positions)
-          for (auto x : p)
+          for (auto& x : p)
             b.extend(x);
         return b;
       }
@@ -980,7 +1101,7 @@ namespace embree
         avector<BBox3fa> bboxes(positions.size());
         for (size_t t=0; t<positions.size(); t++) {
           BBox3fa b = empty;
-          for (auto x : positions[t]) b.extend(x);
+          for (auto& x : positions[t]) b.extend(x);
           bboxes[t] = b;
         }
         return LBBox3fa(bboxes);
@@ -1057,7 +1178,7 @@ namespace embree
       {
         BBox3fa b = empty;
         for (const auto& p : positions)
-          for (auto x : p)
+          for (auto& x : p)
             b.extend(x);
         return b;
       }
@@ -1067,7 +1188,7 @@ namespace embree
         avector<BBox3fa> bboxes(positions.size());
         for (size_t t=0; t<positions.size(); t++) {
           BBox3fa b = empty;
-          for (auto x : positions[t]) b.extend(x);
+          for (auto& x : positions[t]) b.extend(x);
           bboxes[t] = b;
         }
         return LBBox3fa(bboxes);
@@ -1164,7 +1285,7 @@ namespace embree
       {
         BBox3fa b = empty;
         for (const auto& p : positions)
-          for (auto x : p)
+          for (auto& x : p)
             b.extend(x);
         return b;
       }
@@ -1174,7 +1295,7 @@ namespace embree
         avector<BBox3fa> bboxes(positions.size());
         for (size_t t=0; t<positions.size(); t++) {
           BBox3fa b = empty;
-          for (auto x : positions[t]) b.extend(x);
+          for (auto& x : positions[t]) b.extend(x);
           bboxes[t] = b;
         }
         return LBBox3fa(bboxes);
@@ -1281,7 +1402,7 @@ namespace embree
       {
         BBox3fa b = empty;
         for (const auto& p : positions)
-          for (auto x : p)
+          for (auto& x : p)
             b.extend(x);
         return b;
       }
@@ -1291,7 +1412,7 @@ namespace embree
         avector<BBox3fa> bboxes(positions.size());
         for (size_t t=0; t<positions.size(); t++) {
           BBox3fa b = empty;
-          for (auto x : positions[t]) b.extend(x);
+          for (auto& x : positions[t]) b.extend(x);
           bboxes[t] = b;
         }
         return LBBox3fa(bboxes);
@@ -1372,7 +1493,7 @@ namespace embree
       {
         BBox3fa b = empty;
         for (const auto& p : positions)
-          for (auto x : p)
+          for (auto& x : p)
             b.extend(x);
         return b;
       }
@@ -1382,7 +1503,7 @@ namespace embree
         avector<BBox3fa> bboxes(positions.size());
         for (size_t t=0; t<positions.size(); t++) {
           BBox3fa b = empty;
-          for (auto x : positions[t])
+          for (auto& x : positions[t])
             b.extend(x);
           bboxes[t] = b;
         }
@@ -1466,7 +1587,7 @@ namespace embree
       {
         BBox3fa b = empty;
         for (const auto& p : positions)
-          for (auto x : p)
+          for (auto& x : p)
             b.extend(x);
         return b;
       }
@@ -1476,7 +1597,7 @@ namespace embree
         avector<BBox3fa> bboxes(positions.size());
         for (size_t t=0; t<positions.size(); t++) {
           BBox3fa b = empty;
-          for (auto x : positions[t]) b.extend(x);
+          for (auto& x : positions[t]) b.extend(x);
           bboxes[t] = b;
         }
         return LBBox3fa(bboxes);
