@@ -2,6 +2,15 @@
 // SPDX-License-Identifier: Apache-2.0
 
 #include "../common/tutorial/tutorial.h"
+#include "../common/tutorial/benchmark_render.h"
+
+#if defined(EMBREE_SYCL_TUTORIAL)
+#  define NAME "pathtracer_sycl"
+#  define FEATURES FEATURE_RTCORE | FEATURE_SYCL
+#else
+#  define NAME "pathtracer"
+#  define FEATURES FEATURE_RTCORE
+#endif
 
 namespace embree
 {
@@ -14,7 +23,7 @@ namespace embree
   struct Tutorial : public SceneLoadingTutorialApplication
   {
     Tutorial()
-      : SceneLoadingTutorialApplication("pathtracer",FEATURE_RTCORE)
+      : SceneLoadingTutorialApplication(NAME,FEATURES)
     {
       registerOption("spp", [] (Ref<ParseStream> cin, const FileName& path) {
           g_spp = cin->getInt();
@@ -32,7 +41,7 @@ namespace embree
     void postParseCommandLine() override
     {
       /* load default scene if none specified */
-      if (scene->size() == 0 && sceneFilename.size() == 0) {
+      if (scene_empty_post_parse()) {
         FileName file = FileName::executableFolder() + FileName("models/cornell_box.ecs");
         parseCommandLine(new ParseStream(new LineCommentFilter(file, "#")), file.path());
       }
@@ -43,9 +52,9 @@ namespace embree
     {
       ImGui::Checkbox("accumulate",&g_accumulate);
       ImGui::Text("max path length");
-      ImGui::DragInt("",&g_max_path_length,1.0f,1,16);
+      ImGui::DragInt("##max_path_length",&g_max_path_length,1.0f,1,16);
       ImGui::Text("samples per pixel");
-      ImGui::DragInt("",&g_spp,1.0f,1,16);
+      ImGui::DragInt("##samples per pixel",&g_spp,1.0f,1,16);
     }
 #endif
   };
@@ -53,5 +62,8 @@ namespace embree
 }
 
 int main(int argc, char** argv) {
+  if (embree::TutorialBenchmark::benchmark(argc, argv)) {
+    return embree::TutorialBenchmark(embree::renderBenchFunc<embree::Tutorial>).main(argc, argv);
+  }
   return embree::Tutorial().main(argc,argv);
 }
